from datetime import timedelta

from beanie import DeleteRules
from fastapi import APIRouter, Depends, Response
from fastapi.logger import logger

from awesome_sso.exceptions import BadRequest, HTTPException, InternalServerError
from awesome_sso.service.depends import get_sso_user_id, sso_registration, sso_user
from awesome_sso.service.settings import Settings
from awesome_sso.service.user.schema import AccessToken, AwesomeUserType, RegisterModel
from awesome_sso.util.jwt import SYMMETRIC_ALGORITHM, create_token

router = APIRouter(tags=["sso"])


@router.get("/health_check")
def health_check():
    return ["OK"]


@router.post("/register", summary="register user")
async def register(register_model: RegisterModel = Depends(sso_registration)):
    try:
        user = await Settings[AwesomeUserType]().user_model.find_one(  # type: ignore
            Settings[AwesomeUserType]().user_model.email == register_model.email  # type: ignore
        )
        if user is None:
            user = await Settings[AwesomeUserType]().user_model.register(register_model)  # type: ignore
        else:
            raise BadRequest(message="user email %s taken" % register_model.email)
    except HTTPException as e:
        logger.warning(str(e))
        raise e
    except Exception as e:
        logger.warning(str(e))
        raise InternalServerError(message=str(e))
    return user


@router.post("/login", summary="get login access token", response_model=AccessToken)
async def login(user: AwesomeUserType = Depends(sso_user)):
    jwt_payload = {"sso_user_id": str(user.sso_user_id)}
    token = create_token(
        jwt_payload,
        Settings.symmetric_key,
        SYMMETRIC_ALGORITHM,
        expires_delta=timedelta(days=7),
    )
    return AccessToken(access_token=token)


@router.post("/unregister")
async def unregister(sso_user_id: str = Depends(get_sso_user_id)):
    user = await Settings[AwesomeUserType]().user_model.find_one(  # type: ignore
        Settings[AwesomeUserType]().user_model.sso_user_id == sso_user_id  # type: ignore
    )
    if user is None:
        return Response(status_code=200, content="requested user not exist")
    else:
        await user.delete_data()
        await user.delete(link_rule=DeleteRules.DELETE_LINKS)
        return Response(status_code=200, content="user unregistered")



import json
from typing import List

import requests
from pydantic import EmailStr, HttpUrl


class MailGun:
    def __init__(
        self,
        base_url: HttpUrl,
        api_key: str,
    ):
        self.base_url = base_url
        self.api_key = api_key

    def send_simple_message(
        self,
        from_name: str,
        from_email: EmailStr,
        to: List[EmailStr],
        subject: str,
        text: str,
        cc: List[EmailStr] = [],
        bcc: List[EmailStr] = [],
    ):
        return requests.post(
            self.base_url + "/messages",
            auth=("api", self.api_key),
            data={
                "from": "%s <%s>" % (from_name, from_email),
                "to": to,
                "cc": cc,
                "bcc": bcc,
                "subject": subject,
                "text": text,
            },
        )

    def send_template(
        self,
        from_name: str,
        from_email: EmailStr,
        to: List[EmailStr],
        subject: str,
        template: str,
        data: dict,
        cc: List[EmailStr] = [],
        bcc: List[EmailStr] = [],
    ):
        return requests.post(
            self.base_url + "/messages",
            auth=("api", self.api_key),
            data={
                "from": "%s <%s>" % (from_name, from_email),
                "to": to,
                "cc": cc,
                "bcc": bcc,
                "subject": subject,
                "template": template,
                "h:X-Mailgun-Variables": json.dumps(data),
            },
        )



import requests
from beanie import PydanticObjectId

from awesome_sso.service.notification.schema import UserNotificationListResponse
from awesome_sso.service.settings import Settings


async def read_notification(
    sso_user_id: PydanticObjectId, notification_id: PydanticObjectId
) -> UserNotificationListResponse:
    resp = requests.post(
        "%s/notification/read" % Settings.sso_domain,
        params={"user_id": str(sso_user_id), "notification_id": str(notification_id)},
        timeout=5,
    )
    resp.close()
    if resp.status_code / 2 != 100:
        raise RuntimeError(
            "sync notification with sso failed. %s: "
            + str(resp.content.decode("utf-8"))
        )
    return resp.json()



from datetime import datetime, timedelta
from typing import Optional

import jwt

from awesome_sso.util.encode import JSONEncoder

# to get a string like this run:
# openssl rand -hex 32
ASYMMETRIC_ALGORITHM = "RS256"
SYMMETRIC_ALGORITHM = "HS256"


# 生成token
def create_token(
    data: dict, encode_key, algorithm, expires_delta: Optional[timedelta] = None
):
    to_encode = data.copy()
    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(days=15)
    to_encode.update({"exp": datetime.timestamp(expire)})
    encoded_jwt = jwt.encode(
        to_encode,
        encode_key,
        algorithm=algorithm,
        json_encoder=JSONEncoder,
    )
    return encoded_jwt



from motor.motor_asyncio import AsyncIOMotorClient


class MongoDB:
    def __init__(self, host: str, port: int, username: str, password: str):
        self.client = AsyncIOMotorClient(
            host=host, port=port, username=username, password=password
        )



from typing import Generic, Optional, Type

from fastapi.logger import logger
from pydantic import AnyHttpUrl, parse_obj_as

from awesome_sso.service.user.schema import AwesomeUserType


class Settings(Generic[AwesomeUserType]):
    sso_domain: AnyHttpUrl
    service_name: Optional[str] = None
    public_key: str = ""
    private_key: Optional[str] = ""
    symmetric_key: str = ""
    user_model: Type[AwesomeUserType]

    @classmethod
    def init_app(
        cls,
        symmetric_key: str,
        user_model: Type[AwesomeUserType],
        service_name: str,
        public_key: str,
        private_key: Optional[str] = "",
        sso_domain: str = "http://sso-be:3500/api/sso",
    ):
        cls.user_model = user_model
        cls.public_key = public_key
        cls.private_key = private_key
        cls.symmetric_key = symmetric_key
        cls.service_name = service_name
        cls.sso_domain = parse_obj_as(AnyHttpUrl, sso_domain)



from fastapi import FastAPI, Request
from fastapi.logger import logger
from fastapi.responses import JSONResponse

from awesome_sso.exceptions import HTTPException
from awesome_sso.service.route import router


class Service:
    @staticmethod
    def init_app(app: FastAPI):
        app.include_router(router)

        @app.exception_handler(HTTPException)
        async def http_exception_handler(request: Request, exc: HTTPException):
            return JSONResponse(
                status_code=exc.status_code,
                content={"detail": exc.detail, "error_code": exc.error_code},
            )



from typing import Optional


class HTTPException(Exception):
    detail: dict
    status_code: int
    error_code: str

    def __init__(
        self,
        status_code: int,
        message: str,
        error_code: Optional[str] = None,
        args: dict = {},
    ):
        self.status_code = status_code
        self.detail = {"message": message, **args}
        self.error_code = error_code if error_code is not None else str(status_code)


class BadRequest(HTTPException):
    def __init__(self, message, error_code: Optional[str] = None, args: dict = {}):
        super().__init__(
            status_code=400,
            message="Bad Request: %s" % message,
            error_code=error_code,
            args=args,
        )


class Unauthorized(HTTPException):
    def __init__(self, message, error_code: Optional[str] = None, args: dict = {}):
        super().__init__(
            status_code=401,
            message="Unauthorized: %s" % message,
            error_code=error_code,
            args=args,
        )


class NotFound(HTTPException):
    def __init__(self, message, error_code: Optional[str] = None, args: dict = {}):
        super().__init__(
            status_code=404,
            message="Not Found: %s" % message,
            error_code=error_code,
            args=args,
        )


class ResourceGone(HTTPException):
    def __init__(self, message, error_code: Optional[str] = None, args: dict = {}):
        super().__init__(
            status_code=410,
            message="Resource Gone: %s" % message,
            error_code=error_code,
            args=args,
        )


class UnprocessableEntity(HTTPException):
    def __init__(self, message, error_code: Optional[str] = None, args: dict = {}):
        super().__init__(
            status_code=422,
            message="Unprocessable Entity: %s" % message,
            error_code=error_code,
            args=args,
        )


class InternalServerError(HTTPException):
    def __init__(self, message, error_code: Optional[str] = None, args: dict = {}):
        super().__init__(
            status_code=500,
            message="Internal Server Error: %s" % message,
            error_code=error_code,
            args=args,
        )



import requests

from awesome_sso.service.settings import Settings
from awesome_sso.service.user.schema import AwesomeUserType


async def sync_user(user: AwesomeUserType):
    resp = requests.get(
        "%s/user" % Settings.sso_domain,
        params={"user_id": str(user.sso_user_id)},
        timeout=5,
    )
    resp.close()
    if resp.status_code / 2 != 100:
        raise RuntimeError(
            "update vendor info failed: " + str(resp.content.decode("utf-8"))
        )
    sso_user = resp.json()
    user.email = sso_user["email"]
    user.name = sso_user["name"]
    user.line_linked = sso_user["line_linked"]
    services = sso_user["services"]
    services_info = []
    config_values = {}
    for service in services:
        if service["service_name"] == Settings.service_name:
            config_values = {
                config["name"]: config["value"] for config in service["config_values"]
            }
            if "status" in service and service["status"] == "trial_ended":
                config_values["status"] = "trial_ended"
        services_info.append(
            {
                "service_name": service["service_name"],
                "external_domain": service["external_domain"],
            }
        )
    config_values["services"] = services_info
    user.settings = config_values
    await user.save()



from fastapi.logger import logger
from requests import Response


def response_error_check(response: Response):
    if response.status_code / 2 != 100:
        if "json" in str(response.headers.get("content-type")):
            logger.error(response.json())
        else:
            logger.error(response.content.decode("utf-8"))



import glob
import json
from io import BytesIO, StringIO
from logging import Logger
from os import path
from pathlib import Path
from typing import IO, List, Optional

import pandas as pd
from fastapi import UploadFile
from fastapi.logger import logger
from minio import Minio, S3Error
from minio.deleteobjects import DeleteObject

from awesome_sso.exceptions import UnprocessableEntity


class MinioStore:
    bucket: str
    client: Minio
    logger: Logger

    def __init__(
        self,
        host: str,
        bucket: str,
        access_key: str,
        secret_key: str,
        secure: bool = False,
        region: Optional[str] = None,
        logger: Optional[Logger] = None,
    ):
        self.bucket = bucket
        self.client = Minio(
            host,
            access_key=access_key,
            secret_key=secret_key,
            secure=secure,
            region=region,
        )
        self.logger = logger if logger is not None else Logger("minio")
        found = self.client.bucket_exists(self.bucket)
        if not found:
            self.logger.warning("bucket not exist, creating it")
            self.client.make_bucket(self.bucket)
        else:
            self.logger.info("bucket '%s' exists", self.bucket)

    def list_buckets(self):
        """List information of all accessible buckets with text."""
        return [x.name for x in self.client.list_buckets()]

    def list_objects(self, prefix: str = None, recursive: bool = False):
        """Lists object information of a bucket with text."""
        return [
            x.object_name
            for x in self.client.list_objects(
                self.bucket, prefix=prefix, recursive=recursive
            )
        ]

    def fput(self, name: str, file_path: str, exclude_files: List[str] = []):
        """Uploads data from a file/folder to an object in a bucket."""
        if path.isdir(file_path):
            for local_file in glob.glob(file_path + "/**"):
                file_name = Path(local_file).name
                remote_path = path.join(name, file_name)

                if file_name in exclude_files:
                    self.logger.info(f"exclude: {local_file}")
                    continue

                if not path.isfile(local_file):
                    self.fput(remote_path, local_file, exclude_files)
                else:
                    remote_path = path.join(name, local_file[1 + len(file_path) :])
                    self.client.fput_object(self.bucket, remote_path, local_file)
        else:
            self.client.fput_object(self.bucket, name, file_path)

    def put(
        self,
        name: str,
        data: IO,
        length: Optional[int] = None,
        content_type: str = "application/octet-stream",
    ):
        """Uploads data from a stream to an object in a bucket."""
        if not length:
            length = len(data.read())
            data.seek(0)

        self.client.put_object(
            self.bucket, name, data, length, content_type=content_type
        )

    def put_as_json(self, name: str, data: dict):
        """Uploads data from a json to an object in a bucket."""
        data_bytes = json.dumps(data).encode("utf-8")
        data_byte_stream = BytesIO(data_bytes)

        self.put(name, data_byte_stream, content_type="application/json")

    def get(self, name: str):
        """Gets data of an object."""
        return self.client.get_object(self.bucket, name)

    def exists(self, name: str) -> bool:
        """Check if object or bucket exist."""
        try:
            self.client.stat_object(self.bucket, name)
            return True
        except Exception:
            return False

    def remove_dir(self, folder: str):
        """Remove folder."""
        self.logger.warning("removing %s", folder)
        objects_to_delete = self.list_objects(prefix=folder, recursive=True)
        self.logger.warning("Removing: %s", objects_to_delete)
        self.remove_objects(objects_to_delete)

    def remove_object(self, name: str):
        """Remove an object."""
        self.client.remove_object(self.bucket, name)

    def upload_df(self, name: str, data: pd.DataFrame):
        """Uploads data from a pandas dataframe to an object in a bucket."""
        data_bytes = data.to_csv(index=False).encode("utf-8")
        data_byte_stream = BytesIO(data_bytes)

        self.put(name, data_byte_stream, content_type="application/csv")

    def fget_df(
        self,
        file: UploadFile,
        column_types: dict = {},
        date_columns: List[str] = [],
    ) -> Optional[pd.DataFrame]:
        try:
            file_io = StringIO(str(file.file.read(), "utf-8"))
            df = pd.read_csv(file_io, dtype=column_types, parse_dates=date_columns)
            file_io.close()
        except Exception as e:
            self.logger.warning(UnprocessableEntity("unable to read csv %s" % str(e)))
            return None
        return df

    def get_json(self, name: str) -> dict:
        """Gets data of an object and return a json."""
        try:
            file_obj = self.get(name)
        except S3Error as e:
            self.logger.warning(e)
            return {}
        result = json.load(file_obj)
        file_obj.close()
        file_obj.release_conn()
        return result

    def get_df(
        self,
        name: str,
        column_types: dict = {},
        date_columns: List[str] = [],
    ) -> Optional[pd.DataFrame]:
        """Gets data of an object and return a dataframe."""
        try:
            file_obj = self.get(name)
        except S3Error as e:
            self.logger.warning(e)
            return None

        if not date_columns:
            df = pd.read_csv(file_obj, dtype=column_types)
        else:
            df = pd.read_csv(file_obj, parse_dates=date_columns, dtype=column_types)
        file_obj.close()
        file_obj.release_conn()
        return df

    def remove_objects(self, names: list):
        """Remove objects."""
        objects_to_delete = [DeleteObject(name) for name in names]
        for del_err in self.client.remove_objects(self.bucket, objects_to_delete):
            self.logger.warning("Deletion Error: %s", del_err)

    def download(self, name: str, file_path: str):
        """Downloads data of an object to file."""
        self.client.fget_object(self.bucket, name, file_path)



import os
from typing import List

import psutil
import requests
from fastapi.logger import logger

from awesome_sso.service.settings import Settings
from awesome_sso.service.user.schema import ConfigOption, Service, ServiceStatus
from awesome_sso.util.response_error_check import response_error_check


def register_service(
    internal_domain: str, hostname: str, config_options: List[ConfigOption]
):
    internal_domain = internal_domain
    hostname = hostname
    config_options = config_options
    service = Service(
        name=Settings.service_name,
        internal_domain=f"http://{internal_domain}:3500",
        external_domain=f"https://{hostname}",
        status=ServiceStatus.HEALTHY,
        mem_percent=psutil.virtual_memory().percent,
        cpu_percent=psutil.cpu_percent(),
        config_options=config_options,
        user_register_endpoint="/api/sso/register",
        user_unregister_endpoint="/api/sso/unregister",
        user_login_endpoint="/api/sso/login",
    )
    registration_url = Settings.sso_domain + "/register"
    try:
        resp = requests.post(registration_url, json=service.dict(), timeout=5)
        resp.close()
        response_error_check(resp)
    except Exception as e:
        logger.warning("unable to register with sso: %s", str(e))


def unregister_service():
    if os.environ.get("SSO_REGISTER") == "true":
        unregister_url = (
            Settings.sso_domain + "/unregister?service_name=%s" % Settings.service_name
        )
        try:
            resp = requests.post(unregister_url, timeout=5)
            resp.close()
            response_error_check(resp)
        except Exception as e:
            logger.warning("unable to unregister with sso: %s", str(e))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['awesome_sso',
 'awesome_sso.exceptions',
 'awesome_sso.mail',
 'awesome_sso.service',
 'awesome_sso.service.notification',
 'awesome_sso.service.user',
 'awesome_sso.store',
 'awesome_sso.util']

package_data = \
{'': ['*']}

install_requires = \
['APScheduler>=3.8.1,<4.0.0',
 'PyJWT>=2.3.0,<3.0.0',
 'bcrypt>=3.2.0,<4.0.0',
 'beanie>=1.8.0,<2.0.0',
 'cryptography>=36.0.0,<37.0.0',
 'fastapi>=0.70.0,<0.71.0',
 'minio>=7.1.1,<8.0.0',
 'pandas>=1.3.5,<2.0.0',
 'psutil>=5.8.0,<6.0.0',
 'pydantic[email]>=1.8.2,<2.0.0',
 'uvicorn>=0.16.0,<0.17.0']

setup_kwargs = {
    'name': 'awesome-sso',
    'version': '0.2.18',
    'description': 'sso general utility for services connected to sso',
    'long_description': '[![Stable Version](https://img.shields.io/pypi/v/awesome-sso?label=stable)](https://pypi.org/project/awesome-sso/)\n[![tests](https://github.com/MoBagel/awesome-sso/workflows/ci/badge.svg)](https://github.com/MoBagel/awesome-sso)\n[![Coverage Status](https://coveralls.io/repos/github/MoBagel/awesome-sso/badge.svg?branch=develop)](https://coveralls.io/github/MoBagel/awesome-sso)\n\n# Awesome SSO\n\nA library designed to host common components for a cluster of microservices sharing a single sign on.\n\n## Feature\n\n- [x] A common exception class, supporting both status code and custom error code to map to more detailed error message\n  or serve as i18n key.\n- [x] A common FastAPI app for interaction with service, like login ,registration and unregistration.\n- [x] a connector for minio object store.\n- [x] a connector for beanie, a mongo odm compatible with pydantic.\n\n## Usage\n\n### Installation\n\n1. `pip install awesome-sso`\n\n### Exceptions\n\nUsing fast API as example, we may simply throw exception with a proper status code, and an optional error code. We may\nalso supply arbitrary key value in args dict, to help frontend render better error message.\n\n```python\nfrom awesome_sso.exceptions import NotFound\nfrom fastapi import APIRouter\n\nrouter = APIRouter()\n\n\n@router.get(\'/transactions\')\ndef get(id: str):\n    try:\n        obj = find_by_id(id)\n    except Exception as e:\n        raise NotFound(message=\'transaction not found\' % id, error_code=\'A0001\', args={id: id})\n    ...\n```\n\nAnd we may implement a common error handler to convert all these errors to proper response schema\n\n```python\nfrom awesome_sso.exceptions import HTTPException\nfrom fastapi.requests import Request\nfrom fastapi.responses import JSONResponse\n\n\n@app.exception_handler(HTTPException)\nasync def http_exception_handler(request: Request, exc: HTTPException):\n    return JSONResponse(\n        status_code=exc.status_code,\n        content={\n            \'detail\': exc.detail,\n            \'error_code\': exc.error_code,\n        }\n    )\n```\n\nThis would result in a response with status code 404, and body\n\n```json\n{\n  "status_code": 404,\n  "detail": {\n    "message": "transaction not found",\n    "id": "some_id"\n  },\n  "error_code": "A0001"\n}\n```\n\nWith this response, frontend can decide to simply render detail, or map it to detailed message. If error_code "A0001"\ncorrespond to the following i18 n entry\n\n```json\n"error.A0001": {"en-US": "transaction can not be found with supplied {id}: {message}"}\n```\n\nwe may format message accordingly with\n\n```typescript\nerrorMessage = formatMessage({ id: `error.${error.data.error_code}` }, error.data.detail);\n```\n\nNote that error code is not supplied, is default to status code. So it is always safe to simply use error_code in\nfrontend to decide what to render.\n\n### Data Store\n\n#### Minio\n\nrefer to `tests/test_minio.py`\n\n#### Mongo\n\nrefer to `tests/service/test_user.py`\n\n```python\nfrom beanie import init_beanie\nfrom motor.motor_asyncio import AsyncIOMotorClient\nfrom awesome_sso.service.user.schema import AwesomeUser\n\n\ndef init_mongo():\n    settings = YOUR_SETTINGS()\n    models = [AwesomeUser]\n    cli = AsyncIOMotorClient(settings.mongodb_dsn)\n    await init_beanie(\n        database=cli[settings.mongodb_db_name],\n        document_models=models,\n    )\n    for model in models:\n        await model.get_motor_collection().drop()\n        await model.get_motor_collection().drop_indexes()\n```\n\n### Service\n\n#### configure service settings\n\n```python\nfrom awesome_sso.service.settings import Settings\n\nsettings = Settings()\nsettings.init_app(\n    symmetric_key=\'YOUR_SYMMETRIC_KEY\',  # to encode and decode service token\n    public_key=\'YOUR_PUBLIC_KEY\',  # to decode the token signed by sso\n    user_model=USER_MODEL,  # user orm needs to inherit AwesomeUser from `awesome_sso.user.schema`\n    service_name=\'YOUR_SERVICE_NAME\',  # for service discovery, to recognize service\n    sso_domain=\'YOUR_SSO_DOMAIN\',  # for service registration and sync user\n)\n\n```\n\n#### initial service and mount to your application\n\n```python\nfrom awesome_sso.service import Service\nfrom fastapi import FastAPI\n\napp = FastAPI()\nservice = Service()\nservice.init_app(YOUR_FASTAPI_APP)\napp.mount(\'/YOUR/PATH\', YOUR_FASTAPI_APP)\n```\n\nthen open the api doc, you will see the apis in `awesome_sso.service.user.route`\n\n## Development\n\n### Installing Poetry\n\n1. create your own environment for poetry, and simply run: `pip install poetry`\n2. alternatively, you can refer to [poetry\'s official page](https://github.com/python-poetry/poetry)\n3. to be able to use `poe` directly, `pip install poethepoet`\n\n### Contributing\n\n1. project setup: `poetry install`\n2. create your own branch to start developing new feature.\n3. before creating pr, make sure you pass `poe lint` and `./run_test.sh`.\n    - what happened inside `./run_test.sh` is that a minio server is setup for you temporarily, and teardown and unit\n      test is finished.\n    - notice that `poe test` would also work if you already have a minio up and running. You need the following env\n      variable: `MINIO_ACCESS_KEY`, `MINIO_SECRET_KEY`, `MINIO_ADDRESS` upon running `poe test`.\n4. for a list of available poe command, `poe`\n5. after you submit a pr, you should check if pipeline is successful.\n\n### Releasing\n\n1. `poetry version [new_version]`\n2. `git commit -m"Bump version"`\n3. `git push origin develop`\n4. [create new release](https://github.com/MoBagel/awesome-sso/releases/new) on github.\n5. Create release off develop branch, auto generate notes, and review release note. \n6. Publish release\n\n',
    'author': 'Schwannden Kuo',
    'author_email': 'schwannden@mobagel.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/MoBagel/awesome-sso',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional

from beanie import PydanticObjectId
from pydantic import BaseModel


class NotificationType(str, Enum):
    BROADCAST = "broadcast"
    MULTICAST = "multicast"


class Action(str, Enum):
    WEB = "web"
    EMAIL = "email"
    LINE = "line"


class UserNotification(BaseModel):
    id: PydanticObjectId
    message: str
    type: NotificationType
    action: Action = Action.WEB
    service: Optional[str]
    start_at: Optional[datetime]
    end_at: Optional[datetime]
    option: Dict[str, Any] = {}
    read: bool
    created_at: datetime


class UserNotificationListResponse(BaseModel):
    list: List[UserNotification]
    total: int
    page: Optional[int]
    page_size: Optional[int]



import os
from typing import Optional

import jwt
from beanie import PydanticObjectId
from fastapi import Cookie, Depends, Security
from fastapi.logger import logger
from fastapi.security import HTTPAuthorizationCredentials, HTTPBearer
from pydantic import BaseModel

from awesome_sso.exceptions import BadRequest, NotFound, Unauthorized
from awesome_sso.service.settings import Settings
from awesome_sso.service.user.schema import AwesomeUserType, RegisterModel
from awesome_sso.util.constant import MOCK_USER_ID
from awesome_sso.util.jwt import ASYMMETRIC_ALGORITHM, SYMMETRIC_ALGORITHM

security = HTTPBearer()


class JWTPayload(BaseModel):
    sso_user_id: Optional[PydanticObjectId] = None


async def sso_token_decode(
    credentials: HTTPAuthorizationCredentials = Security(security),
) -> dict:
    try:
        jwt_token = credentials.credentials
        payload = jwt.decode(
            jwt_token, Settings.public_key, algorithms=[ASYMMETRIC_ALGORITHM]
        )
        del payload["exp"]
    except Exception as e:
        logger.warning(e)
        raise Unauthorized(message=str(e))
    return payload


async def sso_registration(
    register_model: RegisterModel, payload: dict = Depends(sso_token_decode)
) -> RegisterModel:
    payload["sso_user_id"] = PydanticObjectId(payload["sso_user_id"])
    if payload != register_model.dict():
        logger.warning(payload)
        logger.warning(register_model.dict())
        raise Unauthorized(message="authentication invalid")
    return RegisterModel(**payload)


def get_sso_user_id(payload: dict = Depends(sso_token_decode)) -> PydanticObjectId:
    if "sso_user_id" not in payload:
        logger.warning(payload)
        raise BadRequest("sso id not found")
    return PydanticObjectId(payload["sso_user_id"])


async def sso_user(
    user_sso_user_id: PydanticObjectId = Depends(get_sso_user_id),
) -> AwesomeUserType:
    user = await Settings[AwesomeUserType]().user_model.find_one(
        Settings[AwesomeUserType]().user_model.sso_user_id == user_sso_user_id,
        fetch_links=True,
    )
    if user is None:
        raise NotFound("user not found")
    return user


async def jwt_token_decode(eightpoint: str = Cookie(None)) -> JWTPayload:
    try:
        payload = jwt.decode(
            eightpoint, Settings.symmetric_key, algorithms=[SYMMETRIC_ALGORITHM]
        )
    except Exception as e:
        environment = os.environ.get("ENV")
        if environment is None or environment == "":
            logger.warning("using mock user")
            payload = {"sso_user_id": MOCK_USER_ID}
        else:
            logger.warning(e)
            raise Unauthorized(str(e))

    try:
        jwt_payload = JWTPayload(sso_user_id=PydanticObjectId(payload["sso_user_id"]))
    except Exception as e:
        raise Unauthorized(str(e))
    return jwt_payload


# get from eightpoint cookies
def sso_user_id(
    payload: JWTPayload = Depends(jwt_token_decode),
) -> PydanticObjectId:
    if payload.sso_user_id is None:
        logger.warning(payload)
        raise BadRequest("sso user id not found")
    return payload.sso_user_id


async def get_current_user(
    sso_id: PydanticObjectId = Depends(sso_user_id),
) -> AwesomeUserType:
    user = await Settings[AwesomeUserType]().user_model.find_one(
        Settings[AwesomeUserType]().user_model.sso_user_id == sso_id, fetch_links=True
    )
    if user is None:
        raise NotFound("user not found")
    return user



from enum import Enum
from typing import Any, Dict, List, Optional, Type, TypeVar

from beanie import Document, PydanticObjectId
from pydantic import AnyHttpUrl, BaseModel, EmailStr


class ConfigType(str, Enum):
    STRING = "str"
    INT = "int"


class ConfigConstraint(BaseModel):
    min: Optional[int] = None
    max: Optional[int] = None
    options: Optional[List[str]] = None


class ConfigOption(BaseModel):
    name: str
    description: str
    type: ConfigType
    constraint: ConfigConstraint
    default: Any


class ServiceStatus(str, Enum):
    HEALTHY = "healthy"
    DOWN = "down"
    END_OF_LIFE = "eof"


class Service(BaseModel):
    name: str
    internal_domain: AnyHttpUrl
    external_domain: AnyHttpUrl
    user_register_endpoint: str
    user_unregister_endpoint: str
    user_login_endpoint: str
    status: ServiceStatus = ServiceStatus.DOWN
    mem_percent: float
    cpu_percent: float
    config_options: List[ConfigOption] = []


class AccessToken(BaseModel):
    access_token: str


class RegisterModel(BaseModel):
    name: str
    email: EmailStr
    sso_user_id: PydanticObjectId


AwesomeUserType = TypeVar("AwesomeUserType", bound="AwesomeUser")


class AwesomeUser(Document):
    name: str
    email: EmailStr
    sso_user_id: PydanticObjectId
    settings: Dict[str, Any] = {}
    line_linked: bool = False

    class Collection:
        name = "user"

    @classmethod
    async def register(
        cls: Type[AwesomeUserType], args: RegisterModel
    ) -> AwesomeUserType:
        extra_params = await cls.extra_constructor_params(args)
        return await cls(**args.dict(), **extra_params).create()

    @classmethod
    async def extra_constructor_params(
        cls: Type[AwesomeUserType], args: RegisterModel
    ) -> dict:
        return {}

    async def delete_data(self):
        pass
